﻿//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  Anwendung                                                                           //
//  _____________________________________________________                               //
//                                                                                      //
//  Copyright (c) 2017 Reimar Grasbon													//
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  This software is subjected to the FreeBSD License. Redistribution and use in source //
//  and binary forms, with or without modification, are permitted provided that the     //
//  following conditions are met:                                                       //
//                                                                                      //
//  1. Redistributions of source code must retain the above copyright notice, this list //
//     of conditions and the following disclaimer.                                      //
//                                                                                      //
//  2. Redistributions in binary form must reproduce the above copyright notice, this   //
//     list of conditions and the following disclaimer in the documentation and/or      //
//     other materials provided with the distribution.                                  //
//                                                                                      //
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY //
//  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED           //
//  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  //
//  IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,    //
//  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT  //
//  NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR  //
//  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,   //
//  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)  //
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE          //
//  POSSIBILITY OF SUCH DAMAGE.                                                         //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////

#include <avr/pgmspace.h>
#include "Cy4Net.h"
#include "Cy4TypePlate.h"
#include "AVRChip.h"
#include "Storing.h"
#include "PushMessage.h"
#include "Board.h"
#include "Timer.h"
#include "Application.h"
#include "ToneSignal.h"

//////////////////////////////////////////////////////////////////////////////////////////
//  Daten  ///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
static SInt08              i2cReadAddress     =  -1;
static UInt08              i2cReadCount       =   0;
static UInt08              i2cReadBuffer[20]  = { 0 };
static SInt08              i2cWriteAddress    =  -1;
static UInt08              i2cWriteCount      =   0;
static UInt08              i2cWriteBuffer[20] = { 0 };
static const PushMessage*  currentPushMessage = NULL;   // Adresse im EEP

//////////////////////////////////////////////////////////////////////////////////////////
//  Cy4-Nachrichtenausstatter  ///////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
static void
SetupPushMessage(Cy4Message* message)
{
    PushMessage pushMessage;
    PUSHMESSAGE_EEPLoad(currentPushMessage, &pushMessage);
    PUSHMESSAGE_ToMessage(&pushMessage, message);
}

//////////////////////////////////////////////////////////////////////////////////////////
//  Antwort-Behandler  ///////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
/*static void
OnResponse(const Cy4Message* message)
{

} */

//////////////////////////////////////////////////////////////////////////////////////////
//  Anfrage-Bearbeiter  //////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  Dienst: GpioConfiguration (read write)                                              //
//                                                                                      //
//  Beschreibung: Konfiguriert GPIOs. Jeder GPIO-Pin ist einem Byte zugeordnet und      //
//  kann mit folgenden Werten konfiguriert werden.                                      //
//                                                                                      //
//               'T' = 84 = 54h : Eingabe, Tristate                                     //
//               'P' = 80 = 50h : Eingabe, Pullup                                       //
//               'H' = 72 = 48h : Ausgabe, High                                         //
//               'L' = 76 = 4Ch : Ausgabe, Low                                          //
//               'X' = 88 = 58h : Unverändert                                           //
//                                                                                      //
//  Voreingestellt für GPIOs ist 'P'.                                                   //
//                                                                                      //
//              0   1   2   3   4   5   6   7   8   9                                   //
//            +---+---+---+---+---+---+---+---+---+---+                                 //
//  Belegung: |GA0|GA1|GA2|GA3|GB0|GB4|GC7|GD2|GD3|GD4|                                 //
//            +---+---+---+---+---+---+---+---+---+---+                                 //
//                                                                                      //
//            [GA0] Configuration A0_GPIO: 'T', 'P', 'H', 'L', 'X'                      //
//            [GA1] Configuration A1_GPIO: 'T', 'P', 'H', 'L', 'X'                      //
//            [GA2] Configuration A2_GPIO: 'T', 'P', 'H', 'L', 'X'                      //
//            [GA3] Configuration A3_GPIO: 'T', 'P', 'H', 'L', 'X'                      //
//            [GB0] Configuration B0_GPIO: 'T', 'P', 'H', 'L', 'X'                      //
//            [GB4] Configuration B4_GPIO: 'T', 'P', 'H', 'L', 'X'                      //
//            [GC7] Configuration C7_GPIO: 'T', 'P', 'H', 'L', 'X'                      //
//            [GD2] Configuration D2_GPIO: 'T', 'P', 'H', 'L', 'X'                      //
//            [GD3] Configuration D3_GPIO: 'T', 'P', 'H', 'L', 'X'                      //
//            [GD4] Configuration D4_GPIO: 'T', 'P', 'H', 'L', 'X'                      //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////
static void
OnGpioConfiguration(const Cy4Service* service, Cy4Message* message)
{
    UInt08 data[10];

    for (UInt08 i=0; i<CARDINALITY(data); i++)
    {   switch (BOARD_GpioConfiguration((Gpio)i))
        {   case PIN_T: data[i] = 'T'; break;
            case PIN_P: data[i] = 'P'; break;
            case PIN_L: data[i] = 'L'; break;
            case PIN_H: data[i] = 'H'; break;
            default:    data[i] = 'X';
        }
    }
    CY4SERVICE_TransferPayload(service, message, data);
    if (CY4MESSAGE_Access(message) == CY4MESSAGE_WRITE)
    {   for (UInt08 i=0; i<CARDINALITY(data); i++)
        {   switch (data[i])
            {   case 'T': BOARD_ConfigureGpio((Gpio)i, PIN_T); break;
                case 'P': BOARD_ConfigureGpio((Gpio)i, PIN_P); break;
                case 'L': BOARD_ConfigureGpio((Gpio)i, PIN_L); break;
                case 'H': BOARD_ConfigureGpio((Gpio)i, PIN_H); break;
                case 'X':                                      break;
                default:  CY4MESSAGE_Invalidate(message);
            }
        }
    }
}

//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  Dienst: AccessGPIO (read write)                                                     //
//                                                                                      //
//  Beschreibung: Zugriff auf die GPIOs.                                                //
//                                                                                      //
//              0   1   2   3   4   5   6   7   8   9                                   //
//            +---+---+---+---+---+---+---+---+---+---+                                 //
//  Belegung: |GA0|GA1|GA2|GA3|GB0|GB4|GC7|GD2|GD3|GD4|                                 //
//            +---+---+---+---+---+---+---+---+---+---+                                 //
//                                                                                      //
//            [GA0] A0_GPIO: 1 = High, 0 = Low, 255 = Unverändert                       //
//            [GA1] A1_GPIO: 1 = High, 0 = Low, 255 = Unverändert                       //
//            [GA2] A2_GPIO: 1 = High, 0 = Low, 255 = Unverändert                       //
//            [GA3] A3_GPIO: 1 = High, 0 = Low, 255 = Unverändert                       //
//            [GB0] B0_GPIO: 1 = High, 0 = Low, 255 = Unverändert                       //
//            [GB4] B4_GPIO: 1 = High, 0 = Low, 255 = Unverändert                       //
//            [GC7] C7_GPIO: 1 = High, 0 = Low, 255 = Unverändert                       //
//            [GD2] D2_GPIO: 1 = High, 0 = Low, 255 = Unverändert                       //
//            [GD3] D3_GPIO: 1 = High, 0 = Low, 255 = Unverändert                       //
//            [GD4] D4_GPIO: 1 = High, 0 = Low, 255 = Unverändert                       //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////
static void
OnAccessGPIO(const Cy4Service* service, Cy4Message* message)
{
    UInt08 data[10];

    if (CY4MESSAGE_Access(message) == CY4MESSAGE_READ)
    {   for (UInt08 i=0; i<CARDINALITY(data); i++) data[i] = BOARD_Gpio((Gpio)i);
        CY4SERVICE_TransferPayload(service, message, data);
    }
    else
    {   for (UInt08 i=0; i<CARDINALITY(data); i++) data[i] = 255;
        CY4SERVICE_TransferPayload(service, message, data);
        for (UInt08 i=0; i<CARDINALITY(data); i++)
        {   switch (data[i])
            {   case   0: BOARD_SetGpio((Gpio)i, false); break;
                case   1: BOARD_SetGpio((Gpio)i, true);  break;
                case 255:                                break;
                default:  CY4MESSAGE_Invalidate(message);
            }
        }
    }
}

//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  Dienst: ReadAnalogInput (read only)                                                 //
//                                                                                      //
//  Beschreibung: Liest den ADC-Wert aus.                                               //
//                                                                                      //
//              0   1                                                                   //
//            +---+---+                                                                 //
//  Belegung: |ADH|ADL|                                                                 //
//            +---+---+                                                                 //
//                                                                                      //
//  [ADx] Wert des AD-Konversters: 0 .. 1023.                                           //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////
static void
OnReadAnalogInput(const Cy4Service* service, Cy4Message* message)
{
    UInt08 data[2];

    UInt16 adc = BOARD_AnalogInput();
    data[0] = (UInt08)(adc>>8);
    data[1] = (UInt08)(adc   );
    CY4SERVICE_TransferPayload(service, message, data);
}

//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  Dienst: AccessLeds (read write)                                                     //
//                                                                                      //
//  Beschreibung: Zugriff auf die LEDs.                                                 //
//                                                                                      //
//              0   1   3                                                               //
//            +---+---+---+                                                             //
//  Belegung: |LD1|LD2|LD3|                                                             //
//            +---+---+---+                                                             //
//                                                                                      //
//  [LD1] PWM-Wert LED 1: 0 .. 255 [#]                                                  //
//  [LD2] Zustand LED 2: 0 = Aus, 1 = Ein                                               //
//  [LD2] Zustand LED 3: 0 = Aus, 1 = Ein                                               //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////
static void
OnAccessLeds(const Cy4Service* service, Cy4Message* message)
{
    UInt08 data[3];

    data[0] = BOARD_Led1Pwm();
    data[1] = BOARD_Led(LED_2);
    data[2] = BOARD_Led(LED_3);
    CY4SERVICE_TransferPayload(service, message, data);
    BOARD_SetLed1Pwm(data[0]);
    BOARD_SetLed(LED_2, !!data[1]);
    BOARD_SetLed(LED_3, !!data[2]);
}

//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  Dienst: ReadKeys (read only)                                                        //
//                                                                                      //
//  Beschreibung: Liest den Zustand der Taasten aus.                                    //
//                                                                                      //
//              0   1                                                                   //
//            +---+---+                                                                 //
//  Belegung: |TA1|TA2|                                                                 //
//            +---+---+                                                                 //
//                                                                                      //
//  [TA1] Zustand Taste 1: 0 = geöffnet, 1 = geschlossen                                //
//  [TA2] Zustand Taste 2: 0 = geöffnet, 1 = geschlossen                                //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////
static void
OnReadKeys(const Cy4Service* service, Cy4Message* message)
{
    UInt08 data[2];

    data[0] = BOARD_Key(KEY_1);
    data[1] = BOARD_Key(KEY_2);
    CY4SERVICE_TransferPayload(service, message, data);
}

//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  Dienst: AccessPushMessages (read write)                                             //
//                                                                                      //
//  Info: Zugriff auf die "Push-Nachricht"-Ziele.                                       //
//                                                                                      //
//               0   1   2   3   4   5                                                  //
//             +---+---+---+---+---+---+  6   7   8   9   10  11                        //
//  Belegung:  |NA1|DA1|L12|L11|L10|PA1|+---+---+---+---+---+---+                       //
//             +---+---+---+---+---+---+|NA2|DA2|L22|L21|L20|PA2|                       //
//                                      +---+---+---+---+---+---+                       //
//                                                                                      //
//                                                                                      //
//  [NA1] Push-Nachricht Ziel Taste 1, Netzadresse   : 0 ..     255                     //
//  [DA1] Push-Nachricht Ziel Taste 1, Geräteadresse : 0 ..     255                     //
//  [L1x] Push-Nachricht Ziel Taste 1, Dienstort     : 0 .. FFFFFEh                     //
//  [PA1] Push-Nachricht Ziel Taste 1, Parameter-Byte: 0 ..     255                     //
//  [NA2] Push-Nachricht Ziel Taste 2, Netzadresse   : 0 ..     255                     //
//  [DA2] Push-Nachricht Ziel Taste 2, Geräteadresse : 0 ..     255                     //
//  [L2x] Push-Nachricht Ziel Taste 2, Dienstort     : 0 .. FFFFFEh                     //
//  [PA2] Push-Nachricht Ziel Taste 2, Parameter-Byte: 0 ..     255                     //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////
static void
OnAccessPushMessages(const Cy4Service* service, Cy4Message* message)
{
    UInt08  data[2*6];

    for (UInt08 i=0; i<CARDINALITY(data)/6; i++)
    {   PushMessage  pushMessage;
        STORING_Load(STO_PushMessage[i], &pushMessage);
        UInt08* d = &data[i*6];
        d[0] = CY4ADDRESS_Net(&pushMessage.address);
        d[1] = CY4ADDRESS_Dev(&pushMessage.address);
        d[2] = (UInt08)(pushMessage.location >> 16);
        d[3] = (UInt08)(pushMessage.location >>  8);
        d[4] = (UInt08)(pushMessage.location >>  0);
        d[5] = pushMessage.payload;
    }
    CY4SERVICE_TransferPayload(service, message, data);
    for (UInt08 i=0; i<CARDINALITY(data)/6; i++)
    {   UInt08* d = &data[i*6];
        PushMessage  pushMessage;
        pushMessage.address  = CY4ADDRESS_Create(d[0], d[1]);
        pushMessage.location = ((UInt32)d[2]<<16) + ((UInt16)d[3]<<8) + d[4];
        pushMessage.payload  = d[5];
        STORING_Save(&pushMessage, STO_PushMessage[i]);
    }
}

//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  Dienst: I2CReadAccess (read write)                                                  //
//                                                                                      //
//  Beschreibung: I²C-Lesevorgang: Hier wird die I²C-Adresse und die Anzahl zu lesender //
//  Datenbytes angegeben. Der eigentliche Vorgang wird mittels 'I2CReadExecute' ausge-  //
//  löst. Das Ergebnis kann mittels 'ReadI2CBuffer' abgefragt werden.                   //
//                                                                                      //
//              0   1                                                                   //
//            +---+---+                                                                 //
//  Belegung: |ADR|CNT|                                                                 //
//            +---+---+                                                                 //
//                                                                                      //
//  [ADR] I²C-Adresse des Gerätes: 0 .. 127.                                            //
//  [CNT] Anzahl zu lesender Datenbytes: 0 .. 20.                                       //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////
static void
OnI2CReadAccess(const Cy4Service* service, Cy4Message* message)
{
    UInt08  data[2];

    data[0] = i2cReadAddress;
    data[1] = i2cReadCount;
    CY4SERVICE_TransferPayload(service, message, data);
    i2cReadAddress = data[0];
    i2cReadCount   = data[1];
    if (i2cReadCount > CARDINALITY(i2cReadBuffer))
    {   i2cReadCount = CARDINALITY(i2cReadBuffer);
    }
}

//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  Dienst: I2CReadBuffer (read write)                                                  //
//                                                                                      //
//  Beschreibung: I²C-Lesevorgang: Zugriff auf die gelesenen Datenbytes. Ungenutzte     //
//  Datenbytes haben den Wert 0.                                                        //
//                                                                                      //
//              0   1   2   ...  19                                                     //
//            +---+---+---+-   -+---+                                                   //
//  Belegung: |D00|D01|D02| ... |N19|                                                   //
//            +---+---+---+-   -+---+                                                   //
//                                                                                      //
//  [Nxx] Empfangenes I²C-Datenbyte: 0 .. 255.                                          //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////
static void
OnI2CReadBuffer(const Cy4Service* service, Cy4Message* message)
{
    CY4SERVICE_TransferPayload(service, message, i2cReadBuffer);
}

//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  Dienst: I2CReadExecute (write only)                                                 //
//                                                                                      //
//  Beschreibung: Löst den I²C-Lesevorgang aus. Grundlage ist die mittels 'I2CRead-     //
//  Access' angegebene Adresse und Anzahl zu lesenden Datenbytes. Das Ergebnis kann     //
//  mittels 'I2CReadBuffer' abgefragt werden.                                           //
//                                                                                      //
//              0                                                                       //
//            +---+                                                                     //
//  Belegung: |FST|                                                                     //
//            +---+                                                                     //
//                                                                                      //
//  [FST] Auslöser für einen I²C-Lesevorgang: 5Eh ('^') .                               //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////
static void
OnI2CReadExecute(const Cy4Service* service, Cy4Message* message)
{
    (void)service;
    bool valid = false;
    if (CY4MESSAGE_ConstData(message)[0] == 0x5E)
    {   for (UInt08 i=0; i<CARDINALITY(i2cReadBuffer); i++) i2cReadBuffer[i] = 0;
        valid = BOARD_ReadI2C(i2cReadAddress, i2cReadBuffer, i2cReadCount);
    }
    if (!valid) CY4MESSAGE_Invalidate(message);
}

//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  Dienst: I2CWriteAccess (read write)                                                 //
//                                                                                      //
//  Beschreibung: I²C-Schreibvorgang: Hier wird die I²C-Adresse und die Anzahl zu       //
//  schreibender Datenbytes angegeben. Die Datenbytes müssen durch 'WriteI2CBuffer'     //
//  im Datenpuffer abgelegt sein, der eigentliche Vorgang wird mittels 'I2CWrite-       //
//  Execute' ausgelöst.                                                                 //
//                                                                                      //
//              0   1                                                                   //
//            +---+---+                                                                 //
//  Belegung: |ADR|CNT|                                                                 //
//            +---+---+                                                                 //
//                                                                                      //
//  [ADR] I²C-Adresse des Gerätes: 0 .. 127.                                            //
//  [CNT] Anzahl zu schreibender Datenbytes: 0 .. 20.                                   //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////
static void
OnI2CWriteAccess(const Cy4Service* service, Cy4Message* message)
{
    UInt08  data[2];

    data[0] = i2cWriteAddress;
    data[1] = i2cWriteCount;
    CY4SERVICE_TransferPayload(service, message, data);
    i2cWriteAddress = data[0];
    i2cWriteCount   = data[1];
    if (i2cWriteCount > CARDINALITY(i2cWriteBuffer))
    {   i2cWriteCount = CARDINALITY(i2cWriteBuffer);
    }
}

//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  Dienst: I2CWriteBuffer (read write)                                                 //
//                                                                                      //
//  Beschreibung: I²C-Schreibvorgang: Zugriff auf die zu schreibenden Datenbytes.       //
//                                                                                      //
//              0   1   2   ...  19                                                     //
//            +---+---+---+-   -+---+                                                   //
//  Belegung: |D00|D01|D02| ... |N19|                                                   //
//            +---+---+---+-   -+---+                                                   //
//                                                                                      //
//  [Nxx] Zu schreibendes I²C-Datenbyte: 0 .. 255.                                      //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////
static void
OnI2CWriteBuffer(const Cy4Service* service, Cy4Message* message)
{
    CY4SERVICE_TransferPayload(service, message, i2cWriteBuffer);
}

//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  Dienst: I2CWriteExecute (write only)                                                //
//                                                                                      //
//  Beschreibung: Löst den I²C-Schreibvorgang aus. Grundlage ist die mittels 'I2CWrite- //
//  Access' angegebene Adresse und Anzahl zu schreibender Datenbytes. Die Datenbytes    //
//  müssen zuvor durch 'I2CWriteBuffer' bereitgestellt werden.                          //
//                                                                                      //
//              0                                                                       //
//            +---+                                                                     //
//  Belegung: |FST|                                                                     //
//            +---+                                                                     //
//                                                                                      //
//  [FST] Auslöser für einen I²C-Schreibvorgang: 5Eh ('^') .                            //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////
static void
OnI2CWriteExecute(const Cy4Service* service, Cy4Message* message)
{
    (void)service;
    bool valid = false;
    if (CY4MESSAGE_ConstData(message)[0] == 0x5E)
    {   valid = BOARD_WriteI2C(i2cWriteAddress, i2cWriteBuffer, i2cWriteCount);
    }
    if (!valid) CY4MESSAGE_Invalidate(message);
}

//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  Dienst: DeviceLabel (read write)                                                    //
//                                                                                      //
//  Beschreibung: Benutzerdefinierte Gerätebezeichnung.                                 //
//                                                                                      //
//              0   1   2   ...  59  60                                                 //
//            +---+---+---+-   -+---+---+                                               //
//  Belegung: |N00|N01|N02| ... |N59| 0 |                                               //
//            +---+---+---+-   -+---+---+                                               //
//                                                                                      //
//  [Nxx] Zeichen der allgemeinen Gerätebezeichnung: 0 .. 255 (ASCII/UTF-8)             //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////
static void
OnDeviceLabel(const Cy4Service* service, Cy4Message* message)
{
    UInt08  label[61];

    STORING_Load(STO_DeviceLabel, label);
    CY4SERVICE_TransferPayload(service, message, label);
    label[CARDINALITY(label)-1] = '\0';
    STORING_Save(label, STO_DeviceLabel);
}

//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  Dienst: FactorySettings (write only)                                                //
//                                                                                      //
//  Beschreibung: Zurückstellen aller Einstellungen auf den Ursprungszustand.           //
//                                                                                      //
//              0                                                                       //
//            +---+                                                                     //
//  Belegung: |FST|                                                                     //
//            +---+                                                                     //
//                                                                                      //
//  [FST] Auslöser zur Wiederherstellung der Werksvoreinstellungen: 5Eh ('^').          //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////
static void
OnFactorySettings(const Cy4Service* service, Cy4Message* message)
{
    (void)service;
    if (CY4MESSAGE_ConstData(message)[0] == 0x5E)
    {   STORING_RestoreDefaults();
        BOARD_Reset();
    }
    else
    {   CY4MESSAGE_Invalidate(message);
    }
}

//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  Dienst: SystemLedSignal (read write)                                                //
//                                                                                      //
//  Beschreibung: Zugriff auf die Signalisierung der System-LED.                        //
//                                                                                      //
//              0                                                                       //
//            +---+                                                                     //
//  Belegung: |SYS|                                                                     //
//            +---+                                                                     //
//                                                                                      //
//  [SYS] Signale der System-LED: 0 = Aus, 1 = Ein, 2 = Herzschlag.                     //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////
static void
OnSystemLedSignal(const Cy4Service* service, Cy4Message* message)
{
    if (CY4MESSAGE_Access(message) == CY4MESSAGE_READ)
    {   CY4MESSAGE_Data(message)[0] = BOARD_SysLedSignal();
    }
    else
    {   SysLedSignal sysLedSignal = CY4MESSAGE_ConstData(message)[0];
        BOARD_SetSysLedSignal(sysLedSignal);
        STORING_Save(&sysLedSignal, STO_SysLedSignal);
    }
}

//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  Dienst: ToneMute (read write)                                                       //
//                                                                                      //
//  Beschreibung: Stummschaltung der Tonausgabe.                                        //
//                                                                                      //
//              0                                                                       //
//            +---+                                                                     //
//  Belegung: |TON|                                                                     //
//            +---+                                                                     //
//                                                                                      //
//  [TON] Ton Stumm: 0 = Aus, 1 = Ein.                                                  //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////
static void
OnToneMute(const Cy4Service* service, Cy4Message* message)
{
    if (CY4MESSAGE_Access(message) == CY4MESSAGE_READ)
    {   CY4MESSAGE_Data(message)[0] = BOARD_IsMuted();
    }
    else
    {   bool mute = !!CY4MESSAGE_ConstData(message)[0];
        BOARD_SetMuted(mute);
        STORING_Save(&mute, STO_ToneMuted);
    }
}

//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  Pflichtdienst: CallTypePlate (read only)                                            //
//                                                                                      //
//  Beschreibung: Geräte-Typenschild erfragen.                                          //
//                                                                                      //
//              0   1   2   3   4   5   6                                               //
//            +---+---+---+---+---+---+---+                                             //
//  Belegung: |AT2|AT1|AT0|AVE|BT2|BT1|BT0|                                             //
//            +---+---+---+---+---+---+---+                                             //
//                                                                                      //
//  [ATx] Anwendungstypennummer: Axxxxxh, xxxxx in BCD: 00000 .. 99999                  //
//  [AVE] Anwendungsversion:     01 .. 99, Haupt/Unternummer in BCD (Bsp: 12h = V1.2)   //
//  [BTx] Boardtypennummer:      Bxxxxxh, xxxxx in BCD: 00000 .. 99999                  //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////
static void
OnCallTypePlate(const Cy4Service* service, Cy4Message* message)
{
    static const UInt08  typePlate[][7] PROGMEM = {  { 'C','y','4','C','o','d','e' },
                                                     CY4TYPEPLATE(DEV_APPL_TYPE,
                                                                  DEV_APPL_VERS,
                                                                  DEV_BOARD_TYPE)
                                                  };

    UInt08 j = (UInt08)(CY4MESSAGE_Location(message) - CY4SERVICE_Base(service));
    for (UInt08 i=0; i<CY4MESSAGE_Count(message); i++)
    {   CY4MESSAGE_Data(message)[i] = pgm_read_byte(&typePlate[1][j]);
        j++;
    }
}

//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  Pflichtdienst: EnterApplication (write only)                                        //
//                                                                                      //
//  Beschreibung: Startet die Anwendung neu.                                            //
//                                                                                      //
//              0                                                                       //
//            +---+                                                                     //
//  Belegung: |EAP|                                                                     //
//            +---+                                                                     //
//                                                                                      //
//  [EAP] Auslöser zum (Neu-)Start der Applikation (Reset): 5Eh ('^') .                 //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////
static void
OnEnterApplication(const Cy4Service* service, Cy4Message* message)
{
    (void)service;
    if (CY4MESSAGE_ConstData(message)[0] == 0x5E) BOARD_Reset();
    else                                          CY4MESSAGE_Invalidate(message);
}

//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  Pflichtdienst: EnterProgramLoader (write only)                                      //
//                                                                                      //
//  Beschreibung: Startet das System neu und kehrt im Programmlader-Modul zurück.       //
//                                                                                      //
//              0                                                                       //
//            +---+                                                                     //
//  Belegung: |EPL|                                                                     //
//            +---+                                                                     //
//                                                                                      //
//  [EPL] Auslöser zum Eintritt in den Programmlader: 5Eh ('^') .                       //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////
static void
OnEnterProgramLoader(const Cy4Service* service, Cy4Message* message)
{
    (void)service;
    if (CY4MESSAGE_ConstData(message)[0] == 0x5E) BOARD_EnterPgmLdr();
    else                                          CY4MESSAGE_Invalidate(message);
}

//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  Pflichtdienst: ChangeDeviceAddress (write only)                                     //
//                                                                                      //
//  Beschreibung: Wechsel der Geräteadresse.                                            //
//                                                                                      //
//              0   1                                                                   //
//            +---+---+                                                                 //
//  Belegung: |NET|DEV|                                                                 //
//            +---+---+                                                                 //
//                                                                                      //
//  [NET] Neue Netzadresse:   1 .. 255                                                  //
//  [DEV] Neue Geräteadresse: 1 .. 255                                                  //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////
static void
OnChangeDeviceAddress(const Cy4Service* service, Cy4Message* message)
{
    UInt08  data[2];
    {   Cy4Address address = CY4NET_Address();
        data[0] = CY4ADDRESS_Net(&address);
        data[1] = CY4ADDRESS_Dev(&address);
    }
    CY4SERVICE_TransferPayload(service, message, data);
    {   Cy4Address address = CY4ADDRESS(data[0], data[1]);
        if (CY4ADDRESS_IsBroadcast(&address) || !CY4ADDRESS_IsValid(&address))
        {   CY4MESSAGE_Invalidate(message);
        }
        else
        {   CY4NET_SetAddress(address);
            STORING_SaveCy4SelfAdr(&address);
        }
    }
}

//////////////////////////////////////////////////////////////////////////////////////////
//  Lokale Funktionen   //////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
static void
SendPushMessage(const PushMessage* eepPushMessage)
{
    currentPushMessage = eepPushMessage;
    if (!CY4NET_IsPending(SetupPushMessage, NULL)) CY4NET_Post(SetupPushMessage, NULL);
}

//////////////////////////////////////////////////////////////////////////////////////////
//  Funktionen  //////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
void
APPLICATION_Startup(void)
{
    static const Cy4ServiceItem  servicesTable[] PROGMEM =
    {   //|                 service                  |        handler        |
        //+---------------------------------------.....---+-----------------------+
        { CY4SERVICE(0x700700,  10, CY4SERVICE_READ_WRITE), OnGpioConfiguration   },
        { CY4SERVICE(0x700710,  10, CY4SERVICE_READ_WRITE), OnAccessGPIO          },
        { CY4SERVICE(0x700720,   2, CY4SERVICE_READ_ONLY ), OnReadAnalogInput     },
        { CY4SERVICE(0x700730,   3, CY4SERVICE_READ_WRITE), OnAccessLeds          },
        { CY4SERVICE(0x700740,   2, CY4SERVICE_READ_ONLY ), OnReadKeys            },
        { CY4SERVICE(0x700750,  12, CY4SERVICE_READ_WRITE), OnAccessPushMessages  },
        { CY4SERVICE(0x700800,   2, CY4SERVICE_READ_WRITE), OnI2CReadAccess       },
        { CY4SERVICE(0x700810,  20, CY4SERVICE_READ_WRITE), OnI2CReadBuffer       },
        { CY4SERVICE(0x700890,   1, CY4SERVICE_WRITE_ONLY), OnI2CReadExecute      },
        { CY4SERVICE(0x700900,   2, CY4SERVICE_READ_WRITE), OnI2CWriteAccess      },
        { CY4SERVICE(0x700910,  20, CY4SERVICE_READ_WRITE), OnI2CWriteBuffer      },
        { CY4SERVICE(0x700990,   1, CY4SERVICE_WRITE_ONLY), OnI2CWriteExecute     },
        { CY4SERVICE(0xFFFF00,  61, CY4SERVICE_READ_WRITE), OnDeviceLabel         },
        { CY4SERVICE(0xFFFF40,   1, CY4SERVICE_WRITE_ONLY), OnFactorySettings     },
        { CY4SERVICE(0xFFFF50,   1, CY4SERVICE_READ_WRITE), OnSystemLedSignal     },
        { CY4SERVICE(0xFFFF52,   1, CY4SERVICE_READ_WRITE), OnToneMute            },
        { CY4SERVICE(0xFFFFF0,   7, CY4SERVICE_READ_ONLY ), OnCallTypePlate       },
        { CY4SERVICE(0xFFFFF8,   1, CY4SERVICE_WRITE_ONLY), OnEnterApplication    },
        { CY4SERVICE(0xFFFFFA,   1, CY4SERVICE_WRITE_ONLY), OnEnterProgramLoader  },
        { CY4SERVICE(0xFFFFFC,   2, CY4SERVICE_WRITE_ONLY), OnChangeDeviceAddress }
    };

    CY4NET_AssignServices(servicesTable);
    Cy4Address address = STORING_LoadCy4SelfAdr();
    CY4NET_SetAddress(address);
    {   SysLedSignal  sysLed;
        STORING_Load(STO_SysLedSignal, &sysLed);
        BOARD_SetSysLedSignal(sysLed);
    }
    {   bool muted;
        STORING_Load(STO_ToneMuted, &muted);
        BOARD_SetMuted(muted);
    }
    TONESIGNAL_Start(TONESIGNAL_STARTUP);
}

//////////////////////////////////////////////////////////////////////////////////////////
void
APPLICATION_Proceed(void)
{
    if (BOARD_TakeKey(KEY_1))
    {   TONESIGNAL_Start(TONESIGNAL_BEEP_1);
        SendPushMessage(STORING_AddressOf(STO_PushMessage[0]));
    }
    if (BOARD_TakeKey(KEY_2))
    {   TONESIGNAL_Start(TONESIGNAL_BEEP_2);
        SendPushMessage(STORING_AddressOf(STO_PushMessage[1]));
    }
}

//////////////////////////////////////////////////////////////////////////////////////////
void
APPLICATION_Shutdown(void)
{
}

//////////////////////////////////////////////////////////////////////////////////////////
